/***********************************************
 Blokus Champions League v 0.1
 File : bcl.h
 Description : header file
 Date : 2013.05.18
 Author : Kim Sung-hun shkim@csl.skku.edu
 ***********************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define ROUND 5

/* length of each axis */

#define LENGTH 10

#define BLOCK_NUM 12
#define BLOCK_TYPE 10

/* block's status */

#define UP 0x1
#define RIGHT 0x2
#define DOWN 0x4 
#define LEFT 0x8
#define UP_FLIP 0x10
#define RIGHT_FLIP 0x20
#define DOWN_FLIP 0x40
#define LEFT_FLIP 0x80

/* rotate degree */

#define SINGLE 1	/*90 degree*/
#define DOUBLE 2	/*180 degree*/
#define TRIPLE 3 	/*270 degree*/

/* flip directive */

#define HORIZONTAL	1
#define VERTICAL	2

/* team color */

#define EMPTY 0
#define BLACK 1
#define WHITE 2

extern unsigned int current_team;

#define current_team() current_team + 1
#define turn_exit() current_team = ++current_team % 2

/* current placed in */

#define IN_SET 0
#define IN_STAGE 1

/* error code : return value of place_to() function */

enum {NOERROR=0, OVERLAP, OUT_OF_STAGE};

/* player function return code */

enum {PLAYING=0, OUT_OF_SPACE, OUT_OF_BLOCK}; 

/* type of the blocks */

enum {SQUARE=0, HANDGUN, SHOTGUN, WORM, CROWBAR, CHAIR, CROSS, BOOMERANG, STICK, DOT};

struct point {
	/* (x, y) -> stage[x][y] */
	int x;		/*x-axis of stage*/
	int y;		/*y-axis of stage*/
};

struct block {
	/* struct point is set by place_to function */
	struct point point;		/*left-most, bottom point of block*/
	int block;		/*describing block type by enumeration defined above*/
	int state;		/*UP/RIGHT/DOWN/LEFT... and so on*/
	int place;		/*is this block in stage? -> 0 : in set / 1 : in stage*/
};

struct block_size {
	unsigned int height;
	unsigned int width;
};

extern struct block blockset[2][BLOCK_NUM];
extern struct block_size blocksize[BLOCK_TYPE];

#define get_blocks(color) &blockset[color - 1][0]
#define get_block_size(type) blocksize[type]

extern int stage[LENGTH][LENGTH]; /* LENGTH x LENGTH stage */

#define get_stage() stage

extern struct block *last_block;

#define set_last_block(block) (last_block = block)

/*
 square block

                **
 start point -> ** 
 
 */
static const int square[2][2] = {{1, 1}, {1, 1}};

/*
 handgun block

                **
                *
 start point -> *

 */

static const int handgun[3][2] = {{1, 1}, {1, 0}, {1, 0}};

/*
 shotgun block

				**
				*
				*
 start point ->	*

 */

static const int shotgun[4][2] = {{1, 1}, {1, 0}, {1, 0}, {1, 0}};

/*
 worm block

 start point -> *
 				*
 				*
 				*
 				*
 */

static const int worm[5][1] = {{1}, {1}, {1}, {1}, {1}};

/*
 crowbar block

 				**
 				*
 				*
 				*
 start point -> *
 */

static const int crowbar[5][2] = {{1, 1}, {1, 0}, {1, 0}, {1, 0}, {1, 0}};

/*
 chair block

 				**
 start point -> *
 */

static const int chair[2][2] = {{1, 1}, {1, 0}};

/*
 cross block

				 *
			    ***
 start point ->  *
 be careful!
 start point is white space ' '
 */

static const int cross[3][3] = {{0, 1, 0}, {1, 1, 1}, {0, 1, 0}};

/*
 boomerang block

				*
 				*
 start point -> ***
 */

static const int boomerang[3][3] = {{1, 0, 0}, {1, 0, 0}, {1, 1, 1}};

/*
 stick block 

 start point -> *
 				*
 				*
 */

static const int stick[3][1] = {{1}, {1}, {1}};

/*
 dot block

 start point -> *
 */

static const int dot[1][1] = {1};

struct team_info {
	int win;
	int lose;
	int draw;
	int color;			/* black or white */
	int (*func)(int);	/* function pointer*/
};

static void init_stage()
{
	memset(stage, EMPTY, sizeof(stage));
}

static void init_team(struct team_info *team, int (*func)(int))
{
	team->win = 0;
	team->lose = 0;
	team->draw = 0;
	team->func = func;
}

static void init_game(struct team_info *team, int color)
{
	team->color = color;
}

static void init_blocks()
{
	int i;

	blockset[0][0].block = SQUARE;
	blockset[0][1].block = SQUARE;
	blockset[0][2].block = HANDGUN;
	blockset[0][3].block = SHOTGUN;
	blockset[0][4].block = WORM;
	blockset[0][5].block = WORM;
	blockset[0][6].block = CROWBAR;
	blockset[0][7].block = CHAIR;
	blockset[0][8].block = CROSS;
	blockset[0][9].block = BOOMERANG;
	blockset[0][10].block = STICK;
	blockset[0][11].block = DOT;
	
	blockset[1][0].block = SQUARE;
	blockset[1][1].block = SQUARE;
	blockset[1][2].block = HANDGUN;
	blockset[1][3].block = SHOTGUN;
	blockset[1][4].block = WORM;
	blockset[1][5].block = WORM;
	blockset[1][6].block = CROWBAR;
	blockset[1][7].block = CHAIR;
	blockset[1][8].block = CROSS;
	blockset[1][9].block = BOOMERANG;
	blockset[1][10].block = STICK;
	blockset[1][11].block = DOT;

	for (i = 0;i < BLOCK_NUM;i++)
	{
		blockset[0][i].state = UP;
		blockset[0][i].place = IN_SET;
		blockset[1][i].state = UP;
		blockset[1][i].place = IN_SET;
	}
}

void logging(struct block *, int, char *);

int rotate(struct block *, int degree);

int flip(struct block *, int direction);

int place_to(struct block *, int x, int y);

int get_winner(void);
